#include "stdafx.h"
#include "far.hpp"
#include "dialog.hpp"
#include "msg.hpp"

struct PluginStartupInfo Info;
struct FarStandardFunctions FSF;

void InitDialogItems(const struct InitDialogItem *Init, struct FarDialogItem *Item, int ItemsNumber)
{
  const struct InitDialogItem *PInit = Init;
  struct FarDialogItem *PItem = Item;
  for (int i = 0; i < ItemsNumber; i++, PItem++, PInit++) {
    PItem->Type = PInit->Type;
    PItem->X1 = PInit->X1;
    PItem->Y1 = PInit->Y1;
    PItem->X2 = PInit->X2;
    PItem->Y2 = PInit->Y2;
    PItem->Focus = 0;
    PItem->Selected = 0;
    PItem->Flags = PInit->Flags;
    PItem->DefaultButton = 0;
    lstrcpy(PItem->Data, PInit->Data != -1 ? GetItemName(PInit->Data) : "");
  }
};

BOOL GetDlgItemRect(HANDLE hDlg, int itemID, SMALL_RECT *rect)
{
  struct FarDialogItem DlgItem;

  BOOL bCorrectItem = Info.SendDlgMessage(hDlg, DM_GETDLGITEM, itemID, (long)&DlgItem);

  if (!bCorrectItem || !rect)
    return FALSE;

  switch (DlgItem.Type)
  {
  case DI_TEXT:
    rect->Left   = (SHORT)DlgItem.X1;
    rect->Right  = (SHORT)DlgItem.X2;
    rect->Top    = (SHORT)DlgItem.Y1;
    rect->Bottom = (SHORT)DlgItem.Y2;
    break;

  default:
    Info.SendDlgMessage(hDlg, DM_GETITEMPOSITION, itemID, (long)rect);
  }
  
  if (DlgItem.Type == DI_COMBOBOX)
  {
    FarListInfo listInfo;
    BOOL f = Info.SendDlgMessage(hDlg, DM_LISTINFO, itemID, (long)&listInfo);
    if (f && listInfo.ItemsNumber > 0)
      rect->Right++;
  }

  return TRUE;
}

void SetDlgItemInt(HANDLE hDlg, int itemID, int value)
{
  char buf[128];
  wsprintf(buf, "%d", value);

  SetDlgItemStr(hDlg, itemID, buf);
}

void SetDlgItemStr(HANDLE hDlg, int itemID, const char *value)
{
  char buf[MAXSIZE];
  SMALL_RECT ItemRect;
  FarDialogItem Item;
  struct FarDialogItemData ItemData;

  Info.SendDlgMessage(hDlg, DM_GETDLGITEM, itemID, (long)&Item);
  
  GetDlgItemRect(hDlg, itemID, &ItemRect);

  int ItemWidth = min(ItemRect.Right - ItemRect.Left + 1, sizeof(buf) - 1);

  if (lstrlen(value) > ItemWidth)
  {
    lstrcpyn(buf, value, ItemWidth - 3);
    lstrcat(buf, "...");
  }
  else
  {
    lstrcpyn(buf, value, sizeof(buf));
  }

//  int focusedItem = Info.SendDlgMessage(hDlg, DM_GETFOCUS, itemID, (long)&ItemData);
  
  ItemData.PtrLength = lstrlen(buf);
  ItemData.PtrData = buf;
  Info.SendDlgMessage(hDlg, DM_SETTEXT, itemID, (long)&ItemData);

//  Info.SendDlgMessage(hDlg, DM_SETFOCUS, focusedItem, 0);

}

void ShowDlgHelp(HANDLE hDlg, int *lastItemID,
                 MOUSE_EVENT_RECORD *m_record, 
                 int helpItemID, int firstMsgItemID,
                 struct DlgHelpListBoxItem ListBoxItems[], int ListBoxItemsCount)
{
  SMALL_RECT DlgRect;
  struct FarDialogItem DlgItem;

  Info.SendDlgMessage(hDlg, DM_GETDLGRECT, 0, (long)&DlgRect);

  int itemID = 0;
  BOOL bItemHasHelp = FALSE;
  BOOL bCorrectItem = Info.SendDlgMessage(hDlg, DM_GETDLGITEM, itemID, (long)&DlgItem);
  while (bCorrectItem)
  {
    if (itemID != *lastItemID)
    {
      Info.SendDlgMessage(hDlg, DM_GETDLGITEM, itemID, (long)&DlgItem);
      if (
          (DlgItem.Type == DI_BUTTON || DlgItem.Type == DI_TEXT ||
          DlgItem.Type == DI_CHECKBOX || DlgItem.Type == DI_COMBOBOX ||
          DlgItem.Type == DI_EDIT     || DlgItem.Type == DI_FIXEDIT ||
          DlgItem.Type == DI_LISTBOX || DlgItem.Type == DI_SINGLEBOX)
         )
      {
        SMALL_RECT ItemRect;

        GetDlgItemRect(hDlg, itemID, &ItemRect);

        if ((m_record->dwMousePosition.X >= DlgRect.Left + ItemRect.Left) && 
            (m_record->dwMousePosition.X <= DlgRect.Left + ItemRect.Right) &&
            (m_record->dwMousePosition.Y >= DlgRect.Top + ItemRect.Top) && 
            (m_record->dwMousePosition.Y <= DlgRect.Top + ItemRect.Bottom))
        {
          bItemHasHelp = TRUE;

          if (DlgItem.Type == DI_LISTBOX)
          {
            for (int i = 0; i < ListBoxItemsCount; i++)
            {
              if (itemID == ListBoxItems[i].itemID)
              {
                int CurPos = Info.SendDlgMessage(hDlg, DM_LISTGETCURPOS, ListBoxItems[i].itemID, 0);
                itemID = CurPos + ListBoxItems[i].helpID;
                break;
              }
            }
          }
          break;
        }
      }
    }
    itemID++;
    bCorrectItem = Info.SendDlgMessage(hDlg, DM_GETDLGITEM, itemID, (long)&DlgItem);
  }

  if (bCorrectItem)
  {
    *lastItemID = itemID; 

    long focusedItem = Info.SendDlgMessage(hDlg, DM_GETFOCUS, 0, 0);

    CONSOLE_CURSOR_INFO cursorInfo;
    HANDLE hStdout; 

    hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
    GetConsoleCursorInfo(hStdout, &cursorInfo);

//    long cursorSize = Info.SendDlgMessage(hDlg, DM_GETCURSORSIZE, focusedItem, 0);

    SetDlgItemStr(hDlg, helpItemID, bItemHasHelp ? GetItemHelp(firstMsgItemID + itemID) : "");

    Info.SendDlgMessage(hDlg, DM_SETCURSORSIZE, 
      focusedItem, 
      MAKELONG(cursorInfo.bVisible, cursorInfo.dwSize == 100 ? 99 : cursorInfo.dwSize));
  }
}
