#include "stdafx.h"
#include "options.hpp"
#include "far.hpp"
#include "msg.hpp"
#include "wll_spy.hpp"
#include "common.hpp"

const int CfgW = 70;
const int CfgH = 21;
const int Cfg_DefaultItem = CFGID(cfg_EditDlgWidth);

struct OptionItem OptionsList[opt_last] = {
  OPT_INT,  CFGID(cfg_EditDlgWidth)           , "MainDialogWidth"      , "", CfgW , ""                              , CfgW,
  OPT_STR,  CFGID(cfg_PluginPrefix)           , "PluginPrefix"         , "", FALSE, "spy"                           , FALSE,
                                           
  OPT_BOOL, CFGID(cbCfgWLShowHidden)          , "WLShowHidden"         , "", FALSE, ""                              , TRUE,

  OPT_BOOL, CFGID(cbALSmartSelect)            , "ALSmartSelect"        , "", FALSE, ""                              , TRUE,
  OPT_BOOL, CFGID(cbALSmartEnable)            , "ALSmartEnable"        , "", FALSE, ""                              , TRUE,
                                            
  OPT_BOOL, CFGID(cbCfgSwitchToProcessWnd)    , "SwitchToProcessWnd"   , "", FALSE, ""                              , TRUE,

  OPT_BOOL, CFGID(cbCfgFilterCaseSensitive)   , "FilterCaseSensitive"  , "", FALSE, ""                              , FALSE,
  
  OPT_STR,  CFGID(cfg_EditWLItemFormat)       , "WLItemFormat"         , "", FALSE, "(%classname%) '%windowtext%'"  , FALSE,
  OPT_STR,  CFGID(cfg_EditWLCaptionFormat)    , "WLCaptionFormat"      , "", FALSE, "%hwnd% (%classname%)"          , FALSE,
  OPT_STR,  CFGID(cfg_EditWLHelpFormat)       , "WLHelpFormat"         , "", FALSE, "%hwnd% '%windowtext%'"         , FALSE,
  OPT_STR,  CFGID(cfg_EditHistoryItemFormat)  , "HistoryItemFormat"    , "", FALSE, " %hwnd% (%classname%)"        , FALSE,
};

COptions::COptions(void)
{
}

COptions::~COptions(void)
{
}

void COptions::FillDialog(HANDLE hDlg)
{
  for (int i = 0; i < opt_last; i++)
  {
    switch (OptionsList[i].type)
    {
    case OPT_BOOL:
      DlgItem_SetCheck(Info, hDlg, OptionsList[i].dialogID, theOptions->Value((EnumOptionsList)i)->IntValue);
      break;
    
    case OPT_INT:
      SetDlgItemInt(hDlg, OptionsList[i].dialogID, theOptions->Value((EnumOptionsList)i)->IntValue);
      break;
    
    case OPT_STR:
      SetDlgItemStr(hDlg, OptionsList[i].dialogID, theOptions->Value((EnumOptionsList)i)->StrValue);
      Info.SendDlgMessage(hDlg, DM_EDITUNCHANGEDFLAG, OptionsList[i].dialogID, 1);
      break;
    }
  }
  
  DlgItem_SetFocus(Info, hDlg, Cfg_DefaultItem);
  Info.SendDlgMessage(hDlg, DN_GOTFOCUS, Cfg_DefaultItem, 0);
}

BOOL COptions::SetBool(EnumOptionsList option, BOOL value)
{
  if (OptionsList[option].type == OPT_BOOL)
    OptionsList[option].IntValue = value;
  
  return OptionsList[option].type == OPT_BOOL;
}

BOOL COptions::SetInt(EnumOptionsList option, int value)
{
  if (OptionsList[option].type == OPT_INT)
    OptionsList[option].IntValue = value;
  
  return OptionsList[option].type == OPT_INT;
}

BOOL COptions::SetStr(EnumOptionsList option, char *value)
{
  if (OptionsList[option].type == OPT_STR)
    lstrcpyn(OptionsList[option].StrValue, value, OPT_VALUE_SIZE);
  
  return OptionsList[option].type == OPT_STR;
}

const OptionItem* COptions::Value(EnumOptionsList option)
{
  return &OptionsList[option];
}

long WINAPI COptions::CfgDlgProc(HANDLE hDlg, int Msg, int Param1, long Param2)
{
  switch (Msg) 
  {
  case DN_INITDIALOG:
    Info.SendDlgMessage(hDlg, DM_SETMOUSEEVENTNOTIFY , 1, 0);

    theOptions->Load();
    theOptions->FillDialog(hDlg);
    return FALSE;

  case DN_CTLCOLORDLGITEM:
  {
    int color;
    switch (Param1)
    {
    case CFGID(laCfgHelp):
      color = Info.AdvControl(Info.ModuleNumber, ACTL_GETCOLOR, (void*)COL_HELPBOX);
      return 0x30303030;
    case CFGID(cfg_TextHelp):
      color = Info.AdvControl(Info.ModuleNumber, ACTL_GETCOLOR, (void*)COL_HELPTEXT);
      return color;
    }
    break;
  }

  case DN_GOTFOCUS:
    SetDlgItemStr(hDlg, CFGID(cfg_TextHelp), GetItemHelp(CFGITEM(Param1)));
    return FALSE;

  case DN_BTNCLICK:
    switch (Param1)
    {
    case CFGID(btnCfgDlgLoadDefault):
      {
        char buf[512];
        wsprintf(buf, "%s\n%s", 
          GetMsg(MPluginTitle), 
          GetMsg(MQuestion_LoadDefaultSettings));
        if (Info.Message(Info.ModuleNumber,
          FMSG_ALLINONE|FMSG_MB_YESNO,
          NULL,
          (const char * const *)buf,
          0,0) == 0)
        {
          theOptions->Load(true);
          theOptions->FillDialog(hDlg);
        }
        return FALSE;
      }
    
    case CFGID(btnCfgDlgClose):
    case CFGID(btnCfgDlgSave):
      return FALSE;
    }
    break;

  case DN_MOUSEEVENT:
    {
      MOUSE_EVENT_RECORD m_record = *(MOUSE_EVENT_RECORD*)Param2;
      static int dlgCfgLastHelp = -1;
      ShowDlgHelp(
        hDlg, &dlgCfgLastHelp,
        &m_record, 
        CFGID(cfg_TextHelp), staticCfgDlgBox,
        NULL, 0);
      return TRUE;
    }
    break;

  case DN_CLOSE:
    return (Param1 < 0 || 
            Param1 == CFGID(btnCfgDlgClose) || 
            Param1 == CFGID(btnCfgDlgSave));
    break;
  }
  
  return Info.DefDlgProc(hDlg, Msg, Param1, Param2);
}

int COptions::ShowDialog(void)
{
  struct FarDialogItem CfgDlgItems[cfgDlgCount];
  
  static const struct InitDialogItem CfgItems[] = {
    DI_DOUBLEBOX,   0,       0,        CfgW-1, CfgH-4,  0, staticCfgDlgBox,
      
    DI_TEXT,        2,       1,             0,      0,  0, laCfgDlgWidth,
    DI_FIXEDIT,    24,       1,            26,      0,  DIF_MASKEDIT, -1,

    DI_TEXT,       36,       1,             0,      0,  0, la_PluginPrefix,
    DI_EDIT,       52,       1,        CfgW-3,      0,  0, -1,

    DI_SINGLEBOX,   1,       2,        CfgW-2,      2,  0, -1,

    DI_TEXT,        2,       3,             0,      0,  0, laWLItemsFormat,
    DI_EDIT,       24,       3,        CfgW-3,      0,  0, -1,

    DI_TEXT,        2,       4,             0,      0,  0, laWLCaptionFormat,
    DI_EDIT,       24,       4,        CfgW-3,      0,  0, -1,

    DI_TEXT,        2,       5,             0,      0,  0, laWLHelpFormat,
    DI_EDIT,       24,       5,        CfgW-3,      0,  0, -1,

    DI_TEXT,        2,       6,             0,      0,  0, laHistoryItemFormat,
    DI_EDIT,       24,       6,        CfgW-3,      0,  0, -1,

    DI_SINGLEBOX,   1,       7,        CfgW-2,      6,  0, -1,

    DI_CHECKBOX,    2,       8,             0,      0,  0, cbCfgWLShowHidden,
    DI_CHECKBOX,    2,       9,             0,      0,  0, cbALSmartSelect,
    DI_CHECKBOX,    2,      10,             0,      0,  0, cbALSmartEnable,
    DI_CHECKBOX,    2,      11,             0,      0,  0, cbCfgSwitchToProcessWnd,
    DI_CHECKBOX,    2,      12,             0,      0,  0, cbCfgFilterCaseSensitive,
    
    DI_SINGLEBOX,   1,  CfgH-6,        CfgW-2, CfgH-6,  0, -1,

    DI_BUTTON,      0,  CfgH-5,             0,      0,  DIF_CENTERGROUP, btnCfgDlgSave,
    DI_BUTTON,      0,  CfgH-5,             0,      0,  DIF_CENTERGROUP, btnCfgDlgClose,
    DI_BUTTON,      0,  CfgH-5,             0,      0,  DIF_CENTERGROUP, btnCfgDlgLoadDefault,

    DI_SINGLEBOX,   0,  CfgH-3,        CfgW-1, CfgH-1,  0, laCfgHelp,
    DI_TEXT,        1,  CfgH-2,        CfgW-2, CfgH-2,  DIF_SHOWAMPERSAND, -1,
  };
  InitDialogItems(CfgItems, CfgDlgItems, sizeof(CfgItems) / sizeof(CfgItems[0]));
    
  static const char DlgWidthMask[] = "999";
  CfgDlgItems[CFGID(cfg_EditDlgWidth)].Mask = DlgWidthMask;

  CfgDlgItems[CFGID(btnCfgDlgSave)].DefaultButton = TRUE;

  Load();

  int exitCode;
  exitCode = Info.DialogEx(Info.ModuleNumber, 
    -1, -1, 
    CfgW, CfgH,
    NULL, CfgDlgItems,
    sizeof(CfgDlgItems) / sizeof(CfgDlgItems[0]),
    0, 
    0, 
    COptions::CfgDlgProc, 0);

  if (exitCode == CFGID(btnCfgDlgSave))
  {
    for (int i = 0; i < opt_last; i++)
    {
      switch (OptionsList[i].type)
      {
      case OPT_BOOL:
        OptionsList[i].IntValue = CfgDlgItems[OptionsList[i].dialogID].Selected;
        break;
      case OPT_INT:
        OptionsList[i].IntValue = StrToInt(CfgDlgItems[OptionsList[i].dialogID].Data);
        break;
      case OPT_STR:
        lstrcpyn(OptionsList[i].StrValue, CfgDlgItems[OptionsList[i].dialogID].Data, OPT_VALUE_SIZE);
        break;
      }
    }

    Save();
  }

  return (exitCode == CFGID(btnCfgDlgSave));
}

void COptions::Load(BOOL LoadDefaultValues)
{
  char STRValue[OPT_VALUE_SIZE];
  DWORD DWORDValue;
  DWORD Type, Size;
  HKEY hKey;
  DWORD Disposition;
  if (RegCreateKeyEx(HKEY_CURRENT_USER, PluginRootKey, 0, NULL, 0, KEY_READ, NULL, &hKey, &Disposition) == ERROR_SUCCESS )
  {
    for (int i = 0; i < opt_last; i++)
    {
      switch(OptionsList[i].type)
      {
      case OPT_BOOL:
      case OPT_INT:
        Type = REG_DWORD;
        Size = sizeof(DWORDValue);
        if (!LoadDefaultValues && 
            RegQueryValueEx(hKey, 
                            OptionsList[i].name, 
                            0, 
                            &Type, 
                            (BYTE*)&DWORDValue, 
                            &Size) == ERROR_SUCCESS)
          OptionsList[i].IntValue = DWORDValue;
        else
          OptionsList[i].IntValue = OptionsList[i].IntValueDef;

        break;

      case OPT_STR:
        Type = REG_SZ;
        Size = sizeof(STRValue);
        if (!LoadDefaultValues && 
            RegQueryValueEx(hKey, 
                            OptionsList[i].name, 
                            0, 
                            &Type, 
                            (BYTE*)STRValue, 
                            &Size) == ERROR_SUCCESS)
          lstrcpyn(OptionsList[i].StrValue, (char*)STRValue, OPT_VALUE_SIZE);
        else
          lstrcpyn(OptionsList[i].StrValue, OptionsList[i].StrValueDef, OPT_VALUE_SIZE);

        break;
      }
    }
    RegCloseKey(hKey);
  }
}

void COptions::Save(void)
{
  DWORD Disposition;
  HKEY hKey;

  if (RegCreateKeyEx(HKEY_CURRENT_USER, PluginRootKey, 0, NULL, 0, KEY_WRITE, NULL, &hKey, &Disposition) == ERROR_SUCCESS)
  {
    for (int i = 0; i < opt_last; i++)
    {
      switch(OptionsList[i].type)
      {
      case OPT_BOOL:
      case OPT_INT:
        RegSetValueEx(hKey, OptionsList[i].name, 0, REG_DWORD, (BYTE*)&OptionsList[i].IntValue, sizeof(DWORD));
        break;

      case OPT_STR:
        RegSetValueEx(hKey, OptionsList[i].name, 0, REG_SZ, (BYTE*)OptionsList[i].StrValue, lstrlen(OptionsList[i].StrValue) + 1);
        break;
      }
    }
    RegCloseKey(hKey);
  }
}