#include "stdafx.h"
#include "wndlist.hpp"
#include "far.hpp"
#include "wll_spy.hpp"
#include "dialog.hpp"
#include "options.hpp"
#include "msg.hpp"
#include "filter.hpp"

#define IS_FLAG(val,flag)   (((val)&(flag))==(flag))

struct StyleName STDStyles[] = {
  {  WS_OVERLAPPED,      "WS_OVERLAPPED",   0 },
  {  WS_POPUP,           "WS_POPUP",        0 },
  {  WS_CHILD,           "WS_CHILD",        0 },
  {  WS_MINIMIZE,        "WS_MINIMIZE",     0 },
  {  WS_VISIBLE,         "WS_VISIBLE",      0 },
  {  WS_DISABLED,        "WS_DISABLED",     0 },
  {  WS_CLIPSIBLINGS,    "WS_CLIPSIBLINGS", 0 },
  {  WS_CLIPCHILDREN,    "WS_CLIPCHILDREN", 0 },
  {  WS_MAXIMIZE,        "WS_MAXIMIZE",     0 },
  {  WS_CAPTION,         "WS_CAPTION",      0 },
  {  WS_BORDER,          "WS_BORDER",       0 },
  {  WS_DLGFRAME,        "WS_DLGFRAME",     0 },
  {  WS_VSCROLL,         "WS_VSCROLL",      0 },
  {  WS_HSCROLL,         "WS_HSCROLL",      0 },
  {  WS_SYSMENU,         "WS_SYSMENU",      0 },
  {  WS_THICKFRAME,      "WS_THICKFRAME",   0 },
  {  WS_GROUP,           "WS_GROUP",        0 },
  {  WS_TABSTOP,         "WS_TABSTOP",      0 },
  {  WS_MINIMIZEBOX,     "WS_MINIMIZEBOX",  0 },
  {  WS_MAXIMIZEBOX,     "WS_MAXIMIZEBOX",  0 },
  {0,NULL,0}
};

struct StyleName STDExStyles[] = {
  { WS_EX_DLGMODALFRAME,  "WS_EX_DLGMODALFRAME",   0 },
  { WS_EX_NOPARENTNOTIFY, "WS_EX_NOPARENTNOTIFY",  0 },
  { WS_EX_TOPMOST,        "WS_EX_TOPMOST",         0 },
  { WS_EX_ACCEPTFILES,    "WS_EX_ACCEPTFILES",     0 },
  { WS_EX_TRANSPARENT,    "WS_EX_TRANSPARENT",     0 },
  { WS_EX_MDICHILD,       "WS_EX_MDICHILD",        0 },
  { WS_EX_TOOLWINDOW,     "WS_EX_TOOLWINDOW",      0 },
  { WS_EX_WINDOWEDGE,     "WS_EX_WINDOWEDGE",      0 },
  { WS_EX_CLIENTEDGE,     "WS_EX_CLIENTEDGE",      0 },
  { WS_EX_CONTEXTHELP,    "WS_EX_CONTEXTHELP",     0 },
  { WS_EX_RIGHT,          "WS_EX_RIGHT",           0 },
  { WS_EX_LEFT,           "WS_EX_LEFT",            0 },
  { WS_EX_RTLREADING,     "WS_EX_RTLREADING",      0 },
  { WS_EX_LTRREADING,     "WS_EX_LTRREADING",      0 },
  { WS_EX_LEFTSCROLLBAR,  "WS_EX_LEFTSCROLLBAR",   0 },
  { WS_EX_RIGHTSCROLLBAR, "WS_EX_RIGHTSCROLLBAR",  0 },
  { WS_EX_CONTROLPARENT,  "WS_EX_CONTROLPARENT",   0 },
  { WS_EX_STATICEDGE,     "WS_EX_STATICEDGE",      0 },
  { WS_EX_APPWINDOW,      "WS_EX_APPWINDOW",       0 },
  {0,NULL,0}
};

#if !defined(CS_KEYCVTWINDOW)
  #define CS_KEYCVTWINDOW  0x0004
#endif
#if !defined(CS_NOKEYCVT)
  #define CS_NOKEYCVT      0x0100
#endif

struct StyleName STDClStyles[] = {
  { CS_VREDRAW,           "CS_VREDRAW",          0 },
  { CS_HREDRAW,           "CS_HREDRAW",          0 },
  { CS_KEYCVTWINDOW,      "CS_KEYCVTWINDOW",     0 },
  { CS_DBLCLKS,           "CS_DBLCLKS",          0 },
  { CS_OWNDC,             "CS_OWNDC",            0 },
  { CS_CLASSDC,           "CS_CLASSDC",          0 },
  { CS_PARENTDC,          "CS_PARENTDC",         0 },
  { CS_NOKEYCVT,          "CS_NOKEYCVT",         0 },
  { CS_NOCLOSE,           "CS_NOCLOSE",          0 },
  { CS_SAVEBITS,          "CS_SAVEBITS",         0 },
  { CS_BYTEALIGNCLIENT,   "CS_BYTEALIGNCLIENT",  0 },
  { CS_BYTEALIGNWINDOW,   "CS_BYTEALIGNWINDOW",  0 },
  { CS_GLOBALCLASS,       "CS_GLOBALCLASS",      0 },
  { CS_IME,               "CS_IME",              0 },
  {0,NULL,0}
};

struct SpyWindowInfo wndInfo;
struct SpyFilter wndFilter;
char wndMask[128] = "";

void MkWindowStyle(HANDLE hDlg, DWORD value, StyleName stylesList[], int itemID)
{
  int i;
  int itemsNumber = 0;
  struct FarList farList;
  struct FarListItem *listItems;

  if (!stylesList[0].Length)
    for (i = 0; stylesList[i].Name; i++)
      stylesList[i].Length = lstrlen(stylesList[i].Name);

  for (i = 0; stylesList[i].Name; i++)
  {
    if (IS_FLAG(value, stylesList[i].Flag))
      itemsNumber++;
  }

  if (itemsNumber)
  {
    listItems = new FarListItem[itemsNumber];
    farList.ItemsNumber = itemsNumber;
    farList.Items = listItems;

    int num = 0;
    for (i = 0; stylesList[i].Name; i++)
    {
      if (IS_FLAG(value, stylesList[i].Flag))
      {
        lstrcpy(listItems[num].Text, stylesList[i].Name);
        listItems[num].Flags = LIF_DISABLE;
        if (num == 0)
          listItems[num].Flags |= LIF_SELECTED;
      
        //value &= ~stylesList[i].Flag;
        num++;
      }
    }
  
    Info.SendDlgMessage(hDlg, DM_LISTSET, itemID, (long)&farList);
    delete [] listItems;
  }
  else
  {
    Info.SendDlgMessage(hDlg, DM_LISTDELETE, itemID, NULL);
  }
}

void MkWindowDepends(HANDLE hDlg, HWND hwnd, int itemID)
{
  (void) hwnd;

  int i;
  struct FarList farList;
  struct FarListItem *listItems;

  listItems = new FarListItem[WD_LAST];
  farList.ItemsNumber = WD_LAST;
  farList.Items = listItems;

  for (i = 0; i < WD_LAST; i++)
  {
    wsprintf(listItems[i].Text, "%-10s%08X", GetItemName(i + wndDependOwner), wndInfo.dependsWindows[i]);
    listItems[i].Flags = 0;
  }
  listItems[0].Flags |= LIF_SELECTED;

  Info.SendDlgMessage(hDlg, DM_LISTSET, itemID, (long)&farList);
  delete [] listItems;
}

void MkWindowRects(HANDLE hDlg, HWND hwnd, int itemID)
{
  (void) hwnd;

  int i;
  RECT rect;
  struct FarList farList;
  struct FarListItem *listItems;

  listItems = new FarListItem[WR_LAST];
  farList.ItemsNumber = WR_LAST;
  farList.Items = listItems;

  for (i = 0; i < WR_LAST; i++)
  {
    rect = wndInfo.rects[i];
    
    wsprintf(listItems[i].Text, "(%d,%d)-(%d,%d), %dx%d",
      rect.left, rect.top, rect.right, rect.bottom,
      rect.right - rect.left, rect.bottom - rect.top);

    listItems[i].Flags = 0;
  }
  listItems[0].Flags |= LIF_SELECTED;

  Info.SendDlgMessage(hDlg, DM_LISTSET, itemID, (long)&farList);
  delete [] listItems;
}

BOOL my_strstr(const char * str1, const char * str2, BOOL bCaseSensitive)
{
  BOOL result;

  if (bCaseSensitive)
  {
    result = (strstr(str1, str2) != NULL);
  }
  else
  {
    int len1 = lstrlen(str1);
    int len2 = lstrlen(str2);

    char *buf1 = new char[len1 + 1];
    char *buf2 = new char[len2 + 1];

    lstrcpyn(buf1, str1, len1 + 1);
    lstrcpyn(buf2, str2, len2 + 1);

    CharUpperBuff(buf1, lstrlen(buf1));
    CharUpperBuff(buf2, lstrlen(buf2));

    result = (strstr(buf1, buf2) != NULL);

    delete [] buf1;
    delete [] buf2;
  }

  return result;
}

void SetWindowsListItems(HANDLE hDlg)
{
  int i;
  
  struct FarList WindowsList;
  struct FarListItem *listItems;
  struct FarListItemData *listItemsData;

  char buf[sizeof(listItems->Text)];
  
  theSpyWindows->MakeWindowList(theOptions->Value(opt_WLShowHidden)->IntValue);

  listItems = new FarListItem[theSpyWindows->Count()];
  WindowsList.Items = listItems;

  listItemsData = new FarListItemData[theSpyWindows->Count()];
  
  char *prefix = new char[(theSpyWindows->GetMaxLevel() << 1) + 1];

  int listItemsCount = 0;

  for(i = 0; i < theSpyWindows->Count(); i++)
  {
    SpyWindow item = *theSpyWindows->Items(i);

    SpyWindowInfo info;
    theSpyWindows->GetWindowInfo(item.Window, &info);

    BOOL bCaseSensitive = theOptions->Value(opt_FilterCaseSensitive)->IntValue;
    BOOL InFilterMode = Info.SendDlgMessage(hDlg, DM_GETCHECK, cbUseFilter, 0) == BSTATE_CHECKED;

    if (  (!InFilterMode) ||
          (
              (!wndFilter.isHWND || wndFilter.hwnd == info.hwnd)

           && (!wndFilter.isClassName || my_strstr(info.className, wndFilter.className, bCaseSensitive))
           && (!wndFilter.isWindowText || my_strstr(info.windowText, wndFilter.windowText, bCaseSensitive))

           && (!wndFilter.isThreadID || wndFilter.ThreadID == info.ThreadID)
           && (!wndFilter.isProcessID || wndFilter.ProcessID == info.ProcessID)
          )
       )
    {
      lstrcpy(prefix, "");

      for(int j = 0; j < item.Level; j++) 
      {
        static char TWChars[]  = "   ";

        int k = theSpyWindows->GetTreeChar(item.Index, j, item.Level);
        strncat(prefix, &TWChars[k * 2], 2);
      }
      if (theSpyWindows->GetFormatString(info.hwnd, 
        theOptions->Value(opt_WLItemFormat)->StrValue, buf, sizeof(buf)) &&
        lstrlen(prefix) + lstrlen(buf) + 1 <= sizeof(listItems->Text)
        )
      {
        lstrcpy(listItems[listItemsCount].Text, prefix);
        lstrcat(listItems[listItemsCount].Text, buf);
      }
      else
      {
        lstrcpy(listItems[listItemsCount].Text, GetMsg(MErrorTooLongLine));
      }
      listItems[listItemsCount].Flags = 0;

      listItemsData[listItemsCount].Index = listItemsCount;
      listItemsData[listItemsCount].DataSize = sizeof(DWORD);
      listItemsData[listItemsCount].Data = (void*)(DWORD)info.hwnd;
      listItemsData[listItemsCount].Reserved = 0;

      listItemsCount++;
    }
  }
  
  WindowsList.ItemsNumber = listItemsCount;

  Info.SendDlgMessage(hDlg, DM_ENABLEREDRAW, FALSE, 0);

  Info.SendDlgMessage(hDlg, DM_LISTSET, cbWindowsList, (long)&WindowsList);

  for(i = 0; i < listItemsCount; i++)
    Info.SendDlgMessage(hDlg, DM_LISTSETDATA, cbWindowsList, (long)&listItemsData[i]);

  if (listItemsCount > 0)
    theSpyWindows->GetWindowInfo((HWND)listItemsData[0].Data, &wndInfo);
  else
    ZeroMemory(&wndInfo, sizeof(wndInfo));

  FillDialog(hDlg, &wndInfo, FALSE);

  Info.SendDlgMessage(hDlg, DM_ENABLEREDRAW, TRUE, 0);

  delete [] prefix;
  delete [] WindowsList.Items;
  delete [] listItemsData;
}
